#!/usr/bin/env python
# -*- coding: utf-8 -*-
    
#----------------------------------------------------------------------
# lsfacegtk.py
# Vaculik Filip
# 04/30/2007
#----------------------------------------------------------------------

import sys
from GladeWindow import *
from tools import logger

log = logger.getLogger()

#----------------------------------------------------------------------

class Lsfacegtk(GladeWindow):

    #----------------------------------------------------------------------

    def __init__(self, groups, links):
        '''Prepare instance of GUI part'''
        log.debug("NEW instance of [[[ lsfaceGTK ]]]")
        
        self.groups = groups
        self.links  = links
        
        self.activeGroups = []      # list of active groups
        self.toolButtons  = []      # handles to tool Buttons
        self.greenSemaphore = True  # manage activity of group buttons 
        self.init()
        self.load(self.groups, self.links)

    #----------------------------------------------------------------------

    def init(self):

        filename = 'lsfacegtk.glade'

        widgets = [
            'window1',
            'vbox1',
            'menu_file',
            'menu1',
            'imi_new',
            'imi_open',
            'imi_save',
            'imi_saveas',
            'imi_quit',
            'imi_test',
            'menu_edit',
            'menu2',
            'imagemenuitem6',
            'imagemenuitem7',
            'imagemenuitem8',
            'imagemenuitem9',
            'menu_view',
            'menu4',
            'menu_help',
            'menu3',
            'imagemenuitem10',
            'toolbar1',
            'hpaned1',
            'iconview',
            'textview1',
            'statusbar1',
            'dialog-vbox1',
            ]

        handlers = [
            'on_window1_destroy',
            'imi_quit_activate',
            'imi_test_activate',
            'iconview_drag_begin_cb',
            'iconview_notify_cb',
            'iconview_drag_drop_cb',
            'iconview_drag_end_cb',
            ]

        
        #upgrade main window
        top_window = 'window1'
        GladeWindow.__init__(self, filename, top_window, widgets, handlers)
        self.widgets['window1'].set_icon_from_file('lslogo40.ico')
        
        #upgrade toolbar 
        self.widgets['toolbar1'].set_size_request(0, 60)
        self.addToolbarButton('Show All', separator=True, active=False)
        
        #create model for saving icons // first text then icon
        self.model = gtk.ListStore(str, str, gtk.gdk.Pixbuf) 
        self.widgets['iconview'].set_text_column(1)        # prepare iconview
        self.widgets['iconview'].set_pixbuf_column(2)
        self.widgets['iconview'].connect('selection-changed', self.on_select)
        self.widgets['iconview'].connect('item-activated',       self.on_activate)
        #activate-cursor-item - enter or space
        #item-activated       - doubble click or enter
        #selection changed    - everytime if selected icon is changed to another - by mouse / by keyboard
        
        #create textBufer for textView
        self.textBuffer = gtk.TextBuffer()      
        self.widgets['textview1'].set_editable(False)
        
    #----------------------------------------------------------------------

    def load(self, groups, links): 
        """load data aboat groups and icons"""
        
        for group in groups.groups:
            self.addToolbarButton(label=group.base['name'], icon=group.base['icon'], separator=group.base['separator'], active=group.base['active'])
        
        for link in links.links:
            self.addIcon(self.model, link.base['id'], link.base['name'], link.base['iconpath'] + link.base['iconfile'])
           # for i in range(300):
           #     w.addIcon(link.base['id'], link.base['name'], link.base['iconpath'] + link.base['iconfile'])
         
        self.setIcons(self.model)      # show the icon model in iconView 
        

    def on_window1_destroy(self, *args):
        gtk.main_quit()

    #----------------------------------------------------------------------

    def imi_quit_activate(self, *args):
        gtk.main_quit()
        
    #----------------------------------------------------------------------

    def imi_test_activate(self, *args):
        print "imi test"
        self.rebuildIcons()

    #----------------------------------------------------------------------

    def iconview_drag_begin_cb(self, *args):
        pass

    #----------------------------------------------------------------------

    def iconview_notify_cb(self, *args):
        pass

    #----------------------------------------------------------------------

    def iconview_drag_drop_cb(self, *args):
        pass

    #----------------------------------------------------------------------

    def iconview_drag_end_cb(self, *args):
        pass
    
    
    def addToolbarButton(self, label, icon='lslogo35.xpm', separator=False, active=False):
        iconw = gtk.Image() # icon widget
        iconw.set_from_file(icon)
        
        groupButton = gtk.ToggleToolButton() # create
        groupButton.set_label(label)         # setlabel
        groupButton.set_icon_widget(iconw)   # set icon
      
        groupButton.connect("toggled", self.toolbarClickEvent)

        self.widgets['toolbar1'].insert(groupButton, -1) # place in toolbar 
        
        if separator:                        # if True, create separator 
           separator = gtk.SeparatorToolItem() 
           self.widgets['toolbar1'].insert(separator, -1) 
        
        self.widgets['window1'].show_all()   # make visible
        
        self.toolButtons.append(groupButton) 

        groupButton.set_active(active)       # set if active /! must be after adding to toolbar for good function
           
  
    def toolbarClickEvent(self, tbutton):
        """Manage change group selection on toolbar"""
        log.debug("%s", tbutton.get_label())
        
        if self.greenSemaphore == False:
           return
        self.greenSemaphore = False     
        
        self.activeGroups = [] # clear active groups
        
        # if it is first tool button (select all) mean all other tool buttons will be active = false
        if  tbutton.get_label() == self.toolButtons[0].get_label():
            self.toolButtons[0].set_active(True)
            for button in self.toolButtons[1:]:
                button.set_active(False)
        
        #else if is another button is pressed
        else:
            self.toolButtons[0].set_active(False)
            for button in self.toolButtons[1:]:
                if button.get_active():
                    self.activeGroups.append(  button.get_label()  )
        
            #if all buttons is active false  select first button to active True
            if self.activeGroups == []:
                self.toolButtons[0].set_active(True)
          
 
        log.debug("active groups: %s", self.activeGroups )        
        
        self.rebuildIcons()
        
        self.greenSemaphore = True     
    
    def addIcon(self, model, id, name, iconfile='.'):
        """append icon to  """
        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file(iconfile)
        except:
            pixbuf = gtk.gdk.pixbuf_new_from_file('../../data/lslogo.xpm')
                
        model.append([id, name, pixbuf])
       
    def setIcons(self, model): 
        self.widgets['iconview'].set_model(model)
    
    def getLinkFromIcon(self, icon_view):
        """"From info about selected icon return reference to link -- or None cannot be identified icon"""
        model = self.widgets['iconview'].get_model()  #operate in actual model
        selected = icon_view.get_selected_items()
        if len(selected) == 0: return None
        i = selected[0][0]
        # print model[i][0]
        pos = self.links.listOfIds.index(model[i][0])        
        return self.links.links[ pos ]
        
    
    def on_select(self, icon_view):
        """on sellect icon read its description and show it in text view"""
        link = self.getLinkFromIcon(icon_view)
        if link == None: return
        
        self.textBuffer.set_text(link.base['description'])
        self.widgets['textview1'].set_buffer(self.textBuffer)
        print "-------->>> ", link.base['description']        
    
        
    def on_activate(self, icon_view, args):
        link = self.getLinkFromIcon(icon_view) 
        if link == None: return
        
        file = link.base['path'] + link.base['file']
        
        log.debug("Prepare for running: %s", file ) 
        from tools import launcher
        launcher.runScript(file, parallel='True')
    
    def rebuildIcons(self):
        """Prepare set of incons by selected groups in toolbar"""
       
        # if no one group is selected  - show all links
        if self.activeGroups == []:
            self.setIcons(self.model)
            return
        
        newModel = gtk.ListStore(str, str, gtk.gdk.Pixbuf)          
        
        #for every link       
        for link in self.links.links:
            #print "-------->>> ", link.base['name'] 
            # for every its group
            for group in link.groups:
                #print "-------->>> ",  link.groups
                #print "-------->>> ",  self.activeGroups
                if group in self.activeGroups:
                   #print "----Shot---->>> ", group
                   """@todo: rebuild exchanging icons between models"""
                   self.addIcon(newModel, link.base['id'], link.base['name'], link.base['iconpath'] + link.base['iconfile'])
                   break
        
        self.setIcons(newModel)

       
    def printGroups(self):
        """Print toolbar status about toolbar buttons"""    
        for tbutton in self.toolButtons:
            print tbutton.get_label()
           

    def run(argv):
        gtk.main()

#----------------------------------------------------------------------

def main(argv):
    w = Lsfacegtk()
    w.show()
    gtk.main()

#----------------------------------------------------------------------

if __name__ == '__main__':
    main(sys.argv)
