# -*- coding: utf-8 -*-
"""loading knowns, searching for new, put them into the structure, saving them to config file
"""
import logger, osutils, xmltools
import globals as G

log = logger.getLogger()

def initControl():
    """manage basic loading extensions"""
    log.debug("")
    availableExtensions = findAllExtensions(G.extensionsMainDirectory)
    knownBlips          = loadKnownExtensions(G.knownExtensions)
    
    """@todo: ask user what to do with new plugin / silently load as default"""
    """@todo: identify that new version of extension is present"""
    """@todo: rebuild execute table if environment change """
    """@todo: make  diff availableExtensions - foundedExtensions"""
    allExtensionsBlips = knownBlips + loadNewExtensions(availableExtensions)
    
    lsExecuteTables = {}  # dictionary of references to LSExecuteTable
 
    #sepatare blips by mountpoint and put them in the tables
    for mountpoint in G.listOfMountpoints:
        blips = separateByAttribute( allExtensionsBlips, 'mountpoint', mountpoint) 
        executionTable = LSExcuteTable(mountpoint)
        executionTable.buildFromBlips(blips)
        lsExecuteTables[mountpoint] = executionTable

    return lsExecuteTables

def findAllExtensions(dirs = [G.extensionsMainDirectory]):
    "Search in extensions directory for lsextend.xml files"
    log.debug("%s" % (dirs))
    extensionFiles = osutils.listFilesOnLevel(["extensions"], ["lsextend.xml"], 1)
    log.debug(extensionFiles)
    return extensionFiles

def loadKnownExtensions(src = G.knownExtensions):
    """load data about knowns extensions, automaticly stored in 
       previously saved xml file data structure.
       
       If files not exist, action is skipped

    """
    """@todo:  in comparison with allExtensions:
                          - test if src is available 
                          - put out known extensions
                          - return list of objects LSExtensionBlip
    """
    log.debug("%s" % (src))
    knownExtensions = []
    for path in G.knownExtensions:
        """@todo: make branch that truly load info from files, now is just skiped"""
        osutils.lexists(path)
    return knownExtensions   

def loadNewExtensions(paths):
    """load extension from its lsextend.xml file to data structure"""
    
    extensionBlips = []    
    log.debug("%s" % (paths))
    for file in paths:                                     # for every file - from given path
        blips = xmltools.extensionLoadPharser(file)        # for every blip - from one file 
        for blip in blips:    
            if blip != None:
               """@todo: Rewrite this ugly call"""
               
               extensionBlip = LSExtensionBlip( **blip )
               """@todo: Check if correct data are return"""
               extensionBlip.action = osutils.changeDestinationFile(file, extensionBlip.action) # change path to lsextend.xml file on path to action file
               extensionBlips = extensionBlips + [extensionBlip]

    return extensionBlips 

def separateByAttribute(blips, attribute, value):
    """return list of blips form given blips which 
    contain attribute with equal value
    """
   
    log.debug("No. of blips %d %s" % (len(blips),  attribute + " " + value))
    matchBlips = []
    
    for blip in blips:
        if getattr(blip,attribute) == value:
            matchBlips = matchBlips + [blip]
 
    log.debug("No of matched blips %d" % (len(matchBlips)))    
    return matchBlips

def startExtensions():
#    print startBlips[0].action
#    import subprocess
#    dir = subprocess.Popen(startBlips[0].action, stdout=subprocess.PIPE, shell=True)
#    for line in dir.stdout:
#        print line,
    pass

def saveKnownExtensions(path = G.configSavePath + G.extensionsSaveFile):
    print path
    pass

class LSExtensionBlip:
    """data cointainer of extensions
    """    
    def __init__(self, name, version, priority, mountpoint, position,
                 skipto=0, action="run.py", repeattime=0, parallel=False,
                 active=True
                 ):
        """create a data structure of extension
        
        cyclic - id the action should be revoked cyclic after x time in second (0 mean not repeat)
        """
        log.debug("new instance of LSExtensionBlip")
        self.name       = name
        self.version    = version
        self.priority   = priority    

        self.mountpoint = mountpoint
        self.position   = position
        self.skipto     = skipto
        self.action     = action
        self.repeattime = repeattime
        self.paralel    = parallel
        
        self.active     = active
        
    def getAttribute(self, attribute):
        """return value of attribute or None
        """
        if attribute == "mountpoint":
            return self.mountpoint
        else:
            raise AttributeError, attribute
        
    def __cmp__(self, other):
        """Sorting by position in execution table """
        return cmp(self.position, other.position)    
        
        
class LSExcuteTable:
    """Transform blips to execution table, keep the table"""
    
    def __init__(self, mountpoint):
        log.debug("NEW execution table: %s" % (mountpoint))
        self.mountpoint = mountpoint

    def buildFromBlips(self, blips):
        """build execution table from informations in blips""" 
        log.debug("%s" % (len(blips)))
        blipsByPriority     = self._orderBlipsByPriority(blips)
        self.executionTable = self._orderByPosition(blipsByPriority) 
        print "----",self.mountpoint," -->>>", self.executionTable 

    def buildFromFile(self, file):
        """@todo: read from given file"""
        log.debug("%s" % (file))
        pass
   

    def _orderBlipsByPriority(self, blips):
        """convert blips list to dictionary "{position : action} orrered by priority ascendancy"""
        log.debug("No. of blips %s" % ( len(blips) ))
       
        orderPlan  = []
        sortedList = []   
        
        #blibs to array (priority, reference)
        for blip in blips:
            orderPlan.append([  blip.priority, blip  ])
        
        #sort array
        def _orderByFirst(order):
            return order[0]
        orderPlan.sort(key=_orderByFirst)
    
        # convert ordered array to list
        for blip in orderPlan:
            sortedList = sortedList + [blip[1]]
          
        #print sortedList                
        return sortedList 
    
    def _orderByPosition(self, blips):
        """Order blips and delete blips with same position no.
        """
        #rewrite blips with same [position] 
        indexOfPosition = []
        unicateBlips = []
        for blip in blips:
            #print "-------->>> ", blip.priority 
            if blip.position in indexOfPosition:
                log.warning('Building Execution Table [%s] plugin [%s] REPLACE action on position [%s].' % (self.mountpoint, blip.name, blip.position))
                i = indexOfPosition.index(blip.position)
                del indexOfPosition[i]
                del unicateBlips[i]
                
            unicateBlips.append([int(blip.position), blip.action, blip.paralel])
            indexOfPosition.append(blip.position)
        
 
        from operator import itemgetter
        
        return sorted(unicateBlips, key=itemgetter(0))